using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing; // point structure
using System.Diagnostics;

namespace GenoProDOM
{
  /// <summary>
  /// AutoArrange all individuals or try to.
  /// 
  /// Start with the oldest individual, then go with the decendents, 
  /// if a child is married AND the mate has parent, move another genomap the "family"
  /// when the decendents is all gone, continue with the other individual not linked in another GenoMap.
  /// 
  /// Each individual take 2 unit wide (this is mainly because we want have align often) and the vertical alignment is the birth.
  /// </summary>
  public class AutoArrange
  {
    // map point to individual
    Dictionary<Point, Individual> _Map = new Dictionary<Point, Individual>(); // key: x position/y Generation 


    public AutoArrange(List<Individual> allIndividuals)
    {
      if (allIndividuals.Count ==0) {
        return;
      }
      // start with the oldest individual
      allIndividuals.Sort(SortByBirthDate);



      for (int i = 0; i < allIndividuals.Count; i++) {
        Individual current = allIndividuals[i];
        if (!current.IsPlaced) {
          PlaceIndividualRelatives(current, 0);
        }
      }

    }

    private void PlaceIndividualRelatives(Individual ind, int generation)
    {
      // find the first individual not placed

      // check for wife(s) // TODO check for multiple wife
      Individual mate = GetMate(ind);

      if (mate != null) {
        Individual leftParent;
        if (mate.IsMale) {
          // mate is on the left
          SetIndAtPosition(mate, ind.XPosition - 2, generation, 1);
          leftParent = mate;
          mate.AddLinkedRight(ind);
        } else {
          SetIndAtPosition(mate, ind.XPosition + 2, generation,0);
          leftParent = ind;
          ind.AddLinkedRight(mate);
        }

        // check for childs 
        List<Individual> children = GetChildren(ind);
        int nbChildren = children.Count;
        if (nbChildren > 0) {

          children.Sort(SortByBirthDate);

          // we need to know if some of the child have mate and if some if the mate has parent (move to another GenoMaps);
          int i = 0;
          foreach (Individual child in children) {
            Individual childMate = GetMate(child);
            if (childMate != null && childMate.ParentFamily == null) {
              // since the child mate has no parent it gonna be include in the family tree (no genomaps needed).
              // but we exclude the extremity for instance a single girl that have a boyfriend the boyfriend will not be include under the parent
              if ((i > 0 && i < children.Count - 1) || (i == 0 && child.IsMale) || (i == children.Count - 1 && childMate.IsMale)) {
                nbChildren++;
              }
            }
            i++;
          }

          // Position is simple, 2 squarre (x-axis) per individual centered with both parent; // TODO center with multiple parents
          int iChild = 0;
          Individual previousChild = null;  // to determine the linking issue between the individual
          foreach (Individual child in children) {
            bool isPreviousLinked = false;
            Individual childMate = GetMate(child);
            child.AddLinkedRight(leftParent);

            if (childMate != null && childMate.IsMale) {
              // Mate is on the left
              SetIndAtPosition(childMate, leftParent.XPosition + (2 - nbChildren) + (iChild++ * 2), generation + 1, nbChildren - iChild);
              childMate.AddLinkedRight(child);
              if (previousChild != null) {
                previousChild.AddLinkedRight(childMate);
                isPreviousLinked = true;
              }
            }

            if (previousChild != null && !isPreviousLinked) {
              previousChild.AddLinkedRight(child);
            }

            // set the child position
            SetIndAtPosition(child, leftParent.XPosition + (2 - nbChildren) + (iChild++ * 2), generation + 1, nbChildren - iChild);

            if (childMate != null && !childMate.IsMale) {
              // mate is on the right
              child.AddLinkedRight(mate);
              SetIndAtPosition(childMate, leftParent.XPosition + (2 - nbChildren) + (iChild++ * 2), generation + 1, nbChildren - iChild);
            }

            PlaceIndividualRelatives(child, generation + 1);
          }
        }
      } else {
        // no mate
      }


    } // PlaceIndividualRelatives


    /// <summary>
    /// Set an individual to a position, if not able to, move the individual and others linked
    /// </summary>
    /// <param name="ind">the individual to place</param>
    /// <param name="xPosition">the x position wanted</param>
    /// <param name="generation">the generation (y position)</param>
    /// <param name="iReservePlaceAtRight">number consecutive position need to the right (sibblings or mate)</param>
    private void SetIndAtPosition(Individual ind, int xPosition, int generation, int iReservePlaceAtRight)
    {
      if (!ind.IsPlaced) {

        int offset = 0;
        while (true) {  // the - 1 is to make sure there is 2 space between the inds
FIND_EMPTY_SPOT:
          if (_Map.ContainsKey(new Point(xPosition + offset + 1, generation))) {
            offset += 3;
          } else if (_Map.ContainsKey(new Point(xPosition + offset, generation))) {
            offset += 2;
          } else if (_Map.ContainsKey(new Point(xPosition + offset - 1, generation))) {
            offset += 1;
          } else {
            // great we have a spot check for reserved place
            for (int i = 0; i < iReservePlaceAtRight * 2 + 1; i++) {
              if (_Map.ContainsKey(new Point(xPosition + offset + i, generation))) {
                 offset += (i + 2);
                 goto FIND_EMPTY_SPOT;
              }
            }
            break;
          }
        } // while

        if (offset > 0) {
          // need to move the individual to the right by offset and the other linked individual.
          List<Individual> allLinkedRight = new List<Individual>();
          ind.GetAllLinkedRight(allLinkedRight);

          // remove them from the map
          foreach (Individual indToMoveByOffset in allLinkedRight) {
            _Map.Remove(new Point(indToMoveByOffset.XPosition, indToMoveByOffset.YPosition));
          }

          // move them and them in the map
          foreach (Individual indToMoveByOffset in allLinkedRight) {
            indToMoveByOffset.XPosition += offset;
            _Map.Add(new Point(indToMoveByOffset.XPosition, indToMoveByOffset.YPosition), indToMoveByOffset);
          }
        }

        _Map.Add(new Point(xPosition + offset, generation), ind);
        ind.XPosition = xPosition + offset;
        ind.YPosition = generation;

        ind.IsPlaced = true;
      }

    }



    #region Genealogy Util

    /// <summary>
    /// Find the mate of the individual. TODO return multiple mates
    /// </summary>
    /// <returns>the mate or null if none</returns>
    public Individual GetMate(Individual ind)
    {
      if (ind.ChildFamily == null) {
        return null; // no family
      }
      if (ind.IsMale) {
        return ind.ChildFamily.Female;
      } else {
        return ind.ChildFamily.Male;
      }
    }

    public List<Individual> GetChildren(Individual ind)
    {
      if (ind.ChildFamily == null) {
        return null;
      } else {
        return ind.ChildFamily.Childs;
      }
    }

    #endregion

    #region Sorting

    int SortByBirthDate(Individual inda, Individual indb)
    {
      return DateTime.Compare(inda.Birth, indb.Birth);
    }

    #endregion


  }
}
