using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Xml;

namespace GenoProDOM
{
  /// <summary>
  /// Represents a GenoPro family
  /// </summary>
  public class Family
  {
    private string _Id;

    public string Id
    {
      get
      {
        return _Id;
      }
      set
      {
        _Id = value;
      }
    }

    public string PositionXml
    {
      get
      {
        StringBuilder sb = new StringBuilder();
        int topY = Male.Position.Y - 80;
        if (Childs.Count > 0) {
          sb.AppendFormat("{0}, {1}", (Male.Position.X + Female.Position.X) / 2, topY);

        } else {
          sb.AppendFormat("{0}, {1}", Male.Position.X, topY);
        }

        sb.Append("<Top>");

        sb.AppendFormat("<Left>{0}, {1}</Left>", Math.Min(Male.Position.X, Female.Position.X), topY);
        sb.AppendFormat("<Right>{0}, {1}</Right>", Math.Max(Female.Position.X, Male.Position.X), topY);
        sb.Append("</Top>");

        if (Childs.Count > 0) {
          sb.Append("<Bottom>");
          topY -= 15;

          int left = Childs[0].Position.X;
          int right = Childs[Childs.Count - 1].Position.X;
          if (left > right) {
            int tmp = left;
            left = right;
            right = tmp;
          }

          sb.AppendFormat("<Left>{0}, {1}</Left>", left, topY);
          sb.AppendFormat("<Right>{0}, {1}</Right>", right, topY);

          sb.Append("</Bottom>");
        }
        return sb.ToString();
      }

    }



    public List<Individual> Childs = new List<Individual>();

    // create a new family from 2 individuals
    public Family(string id, Individual mate1, Individual mate2)
    {
      _Id = id;
      Debug.Assert(mate1.ChildFamily == null);
      Debug.Assert(mate2.ChildFamily == null);

      if (mate1.IsMale) {
        Male = mate1;
        Female = mate2;
      } else {
        Male = mate2;
        Female = mate1;
      }
      Male.ChildFamily = this;
      Female.ChildFamily = this;
    }

    public Family(XmlNode familyNode)
    {
      _Id = familyNode.Attributes["ID"].Value;
    }

    public Individual Male;
    public Individual Female;

    public override string ToString()
    {
      return string.Format("fam{0}", _Id.ToString().PadLeft(5, '0'));
    }

    public Individual GetMateOf(Individual mate)
    {
      Debug.Assert(mate == Male || mate == Female);
      if (Male == mate) {
        return Female;
      } else {
        return Male;
      }
    }
  }

}
