// Create by Jean-Claude Morin for GenoPro - Everyone can copy / modify / distribute without any restrictions.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;

using GenoProDOM; // very small library to manipulate the object

namespace GenoProAutomationReadWrite
{
  public partial class FrmGenoProAutomation : Form
  {
    public FrmGenoProAutomation()
    {
      InitializeComponent();
      System.Windows.Forms.Control.CheckForIllegalCrossThreadCalls = false;
    }

    private GenoPro.DocumentClass m_GenoProDocument; // This is the reference to the GenoPro Document

    private void cmdLaunchGenoPro_Click(object sender, EventArgs e)
    {
      CreateGenoProInstanceIfNeeded();

      // Send XML ... an empty document
      m_GenoProDocument.SetTextXML(@"<?xml version='1.0' encoding='UTF-8'?><GenoPro></GenoPro>");


      //m_GenoProDocument.SetTextXML(@"<?xml version='1.0' encoding='UTF-8'?><GenoPro></GenoPro>");
      m_GenoProDocument.ShowWindow(); // popup the application to windows desktop
      cmdAutoArrange.Enabled = true;
      cmdTest.Enabled = true;
      cmdUpdate.Enabled = true;
      txtXmlData.Enabled = true;

    }

    /// <summary>
    /// Create a reference to the GenoPro instance. Not that we CANNOT link to an already openned document, this is a know issue that will be solve in GenoPro Gamma
    /// </summary>
    private void CreateGenoProInstanceIfNeeded()
    {
      try {
        if (m_GenoProDocument == null) {
          m_GenoProDocument = new GenoPro.DocumentClass(); // create instance
          m_GenoProDocument.EnableMultiUser(1); // this is a option to make sure every object have an unique id (such as label and pedigree links)

          // **** Register to events we want ****
          // when the user save the document
          m_GenoProDocument.OnDocumentSaving += new GenoPro.IGenoDocumentEvents_OnDocumentSavingEventHandler(m_GenoProDocument_OnDocumentSaving);
          // when the user quit GenoPRo or close the document
          m_GenoProDocument.OnDocumentClosing += new GenoPro.IGenoDocumentEvents_OnDocumentClosingEventHandler(m_GenoProDocument_OnDocumentSaving);
        }
      } catch (Exception ex) {
        MessageBox.Show(ex.Message, ex.GetType().ToString());
      }

    }

    /// <summary>
    /// This method is called when the user save (note that the event must be attached before: function CreateGenoProInstanceIfNeeded())
    /// </summary>
    void m_GenoProDocument_OnDocumentSaving()
    {
      // retrieve the entire XML from GEnoPro
      string data = m_GenoProDocument.GetTextXML();
      txtXmlData.Text = data;
      cmdUpdate.Enabled = false;
    }

    #region AutoArrange

    /// <summary>
    /// Sort of experimental auto-arrange base on birth-date and generation
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void cmdAutoArrange_Click(object sender, EventArgs e)
    {
      try {
        string xml = m_GenoProDocument.GetTextXML();

        // perform autoarrange
        XmlDocument xmlParser = new XmlDocument();
        xmlParser.LoadXml(xml);

        XmlNodeList nodesIndividual = xmlParser.SelectNodes(@"GenoPro/Individuals/Individual");

        Dictionary<string, Individual> inds = new Dictionary<string, Individual>(nodesIndividual.Count);
        foreach (XmlNode nodeInd in nodesIndividual) {
          Individual ind = new Individual(nodeInd);
          inds.Add(ind.Id, ind);
        }

        XmlNodeList nodesFamilies = xmlParser.SelectNodes(@"GenoPro/Families/Family");
        Dictionary<string, Family> fams = new Dictionary<string, Family>(nodesFamilies.Count);
        foreach (XmlNode nodeFam in nodesFamilies) {
          Family fam = new Family(nodeFam);
          fams.Add(fam.Id, fam);
        }

        XmlNodeList nodesFound = xmlParser.SelectNodes(@"GenoPro/PedigreeLinks/PedigreeLink");
        foreach (XmlNode nodePedigree in nodesFound) {
          string idIndividual = nodePedigree.Attributes["Individual"].Value;
          string idFamily = nodePedigree.Attributes["Family"].Value;
          Individual ind = inds[idIndividual];
          Family fam = fams[idFamily];
          bool isParent = nodePedigree.Attributes["PedigreeLink"].Value == "Parent";
          if (isParent) {
            if (ind.IsMale && fam.Male == null) {
              fam.Male = ind;
            } else {
              fam.Female = ind;
            }
            ind.ChildFamily = fam;
          } else {
            // is a child
            fam.Childs.Add(ind);
            ind.ParentFamily = fam;
          }
        }
        Individual[] individualArray = new Individual[inds.Count];
        inds.Values.CopyTo(individualArray, 0);
        List<Individual> allInd = new List<Individual>(inds.Values);
        AutoArrange auto = new AutoArrange(allInd);

        // reassign the position in the xml
        foreach (XmlNode nodeInd in nodesIndividual) {
          string idInd = nodeInd.Attributes["ID"].Value;
          Individual ind = inds[idInd];
          XmlElement position = (XmlElement)nodeInd.SelectSingleNode("Position");
          position.RemoveAttribute("BoundaryRect"); // needed?
          position.InnerText = ind.PositionXml;
        }

        foreach (XmlNode nodeFam in nodesFamilies) {
          string idFam = nodeFam.Attributes["ID"].Value;
          Family fam = fams[idFam];
          XmlElement position = (XmlElement)nodeFam.SelectSingleNode("Position");
          position.InnerXml = fam.PositionXml;
        }

        xml = xmlParser.InnerXml;
        txtXmlData.Text = xml;
        cmdUpdate.Enabled = false;
        m_GenoProDocument.SetTextXML(xml);
      } catch (Exception ex) {
        MessageBox.Show("Error: " + ex.Message);
        cmdAutoArrange.Enabled = false;
      }
    }

    #endregion

    private void txtXmlData_TextChanged(object sender, EventArgs e)
    {
      cmdUpdate.Enabled = true;
    }

    private void cmdUpdate_Click(object sender, EventArgs e)
    {
      m_GenoProDocument.SetTextXML(txtXmlData.Text);
      cmdUpdate.Enabled = false;
    }

    private void cmdTest_Click(object sender, EventArgs e)
    {
      txtXmlData.Text = @"
 <?xml version='1.0' encoding='unicode'?>
<GenoPro>
  <Software>
    <Version>2.0.1.3</Version>
  </Software>
<Global>
  <ActiveGenoMap>My GenoMap</ActiveGenoMap>
</Global>
<GenoMaps>
<GenoMap Name=""My GenoMap"" />
</GenoMaps>
<Individuals>
<Individual ID=""i1"">
  <Name>
    <Display>Dad</Display>
    <First>Dad</First>
  </Name>
  <Position>-255,0</Position>
  <Gender>M</Gender>
  <Birth>
    <Date>1900</Date>
  </Birth>
</Individual>
<Individual ID=""i2"">
  <Name>
    <Display>Mom</Display>
    <First>Mom</First>
  </Name>
  <Position>-105,0</Position>
  <Gender>F</Gender>
  <Birth>
    <Date>1901</Date>
  </Birth>
</Individual>
<Individual ID=""i3"">
  <Name>
    <Display>Son</Display>
    <First>Son</First>
  </Name>
  <Position>-180,-140</Position>
  <Gender>M</Gender>
  <Birth>
    <Date>1950</Date>
  </Birth>
</Individual>
</Individuals>
<Families>
<Family ID=""f1"">
  <Position>-255,-60
    <Top>
      <Left>-255,-60</Left>
      <Right>-105,-60</Right>
    </Top>
  </Position>
  <Relation>Marriage</Relation>
</Family>
</Families>
<PedigreeLinks>
<PedigreeLink ID=""lnka"" PedigreeLink=""Parent"" Family=""f1"" Individual=""i1""/>
<PedigreeLink ID=""lnkb"" PedigreeLink=""Parent"" Family=""f1"" Individual=""i2""/>
<PedigreeLink ID=""lnkc"" PedigreeLink=""Biological"" Family=""f1"" Individual=""i3""/>
</PedigreeLinks>
<Labels>
<Label ID=""lbl"">
  <Position Width=""270"" Height=""50"">-300,110</Position>
  <Text Padding=""10"">Congradulation you have inject XML to GenoPro</Text>
</Label>
</Labels>
</GenoPro>";

      cmdUpdate_Click(sender, e);
    }

    private void cmdLoadExisting_Click(object sender, EventArgs e)
    {
      OpenFileDialog open = new OpenFileDialog();
      if (open.ShowDialog() == DialogResult.OK) {
        try {

          XmlDocument xml = GenoProGnoFile.LoadGenoDocument(open.FileName);
          txtXmlData.Text = xml.InnerXml;
          cmdUpdate_Click(sender, e);

        } catch (Exception) {
          MessageBox.Show("Cannot open file - THIS CODE DO NOT SUPPORT version 1.x file or the fast save files of GenoPro");
        }

      }
    }
  }
}