using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using ICSharpCode.SharpZipLib.Zip;

namespace GenoProAutomationReadWrite
{
  class GenoProGnoFile
  {
    #region IO

    /// <summary>
    /// Load a geno document and return the XML
    /// </summary>
    /// <param name="path">path of the geno file (.gno)</param>
    /// <returns>The xml document inside the geno file</returns>
    public static XmlDocument LoadGenoDocument(string filename)
    {
      if (!System.IO.File.Exists(filename)) {
        return null;
      }
      using (ZipInputStream s = new ZipInputStream(File.OpenRead(filename))) {
        ZipEntry theEntry;
        // loop the file in the zip archive
        XmlDocument xmlDoc = new XmlDocument();
        while ((theEntry = s.GetNextEntry()) != null) {
          if (theEntry.IsFile && string.Compare(theEntry.Name, "Data.xml", true) == 0) {
            xmlDoc.Load(s);
            break;
          }
        }
        s.Close();
        return xmlDoc;
      }
    }

    /// <summary>
    /// Save an XML document a the gno format (zipped)
    /// </summary>
    /// <param name="xml">the xml document</param>
    /// <param name="filename">the name of the file to be saved</param>
    public static void SaveGenoDocument(XmlDocument xml, string filename)
    {
      MemoryStream xmlStream = new MemoryStream();
      xml.Save(xmlStream);

      using (ZipOutputStream s = new ZipOutputStream(File.OpenWrite(filename))) {
        ZipEntry entry = new ZipEntry("Data.xml");
        entry.DateTime = DateTime.Now;
        entry.Size = xmlStream.Length;
        s.PutNextEntry(entry);

        xmlStream.Seek(0, SeekOrigin.Begin);
        byte[] buffer = new byte[4096];
        int read = 0;
        while ((read = xmlStream.Read(buffer, 0, buffer.Length)) > 0) {
          s.Write(buffer, 0, read);
        }
        s.Finish();
        s.Close();
      }

    }

    #endregion

  }
}
